//  Copyright 2020 SILINES. Released under GPLv3 in Russia.
//---------------------------------------------------------------------------------------------------
//  Пример программы работы с устройством RODOS-3. Версия 1.03
//---------------------------------------------------------------------------------------------------
//  www.silines.ru
//---------------------------------------------------------------------------------------------------
#define R3_VID 0x20A0
#define R3_PID 0x4173
//---------------------------------------------------------------------------------------------------
#include <iostream>
#include <stdio.h>
#include <cstring>
#include <unistd.h>
#include <getopt.h>
#include <string>
#include <memory>
#include <inttypes.h>
#include "RODOS3.h"
using namespace std;
//---------------------------------------------------------------------------------------------------
struct flags_t
{
    bool no_flags;
    bool on;
    bool off;
    bool all;
    bool read;
    uint32_t device_id;
    
    uint16_t pulse_delay;
    uint8_t pulse_width;

    void init (void);
};
//---------------------------------------------------------------------------------------------------
int main(int argc, char* argv[])
{
    std::ios::sync_with_stdio(false);

    int MAIN_RESULT = 1;
    flags_t flags;
    flags.init();

    //  Разбор переданных параметров
    const char* short_options = "hi:nfrap:d:";

        const struct option long_options[] = {
            {"help",no_argument,NULL,'h'},
            {"id",required_argument,NULL,'i'},
            {"on",no_argument,NULL,'n'},
            {"off",no_argument,NULL,'f'},
            {"read",no_argument,NULL,'r'},
            {"all",no_argument,NULL,'a'},
            {"pulse",required_argument, NULL, 'p'},
            {"delay",required_argument, NULL, 'd'},
            {NULL,0,NULL,0}
        };

        int rez;
        int option_index;

        while ((rez=getopt_long(argc,argv,short_options,long_options,&option_index))!=-1)
        {
            switch(rez){
                case 'h': {
                    cout << " Используйте параметры\n"
                            "   --help,  -h  - вызов справки\n"
                            "   --id,    -i  - обращение к RODOS-3 с соответствующим ID\n"
                            "   --all,   -a  - для выполнения на всех RODOS-3\n"
                            "   --on,    -n  - включить реле (замкнуть N.O. и COM)\n"
                            "   --off,   -f  - выключить реле (замкнуть N.С. и COM)\n"
                            "   --read,  -r  - считать текущее состояние реле\n"
                            "   --pulse, -p  - отправить на реле импульс указанной длительности; допустимые значения 1...255\n"
                            "   --delay, -d  - задержка перед передачей импульса на реле; используется совместно с ключем --pulse; допустимые значения 1...65535\n"
                            "   Запуск без параметров - получение списка всех устройств и их ID\n"
                         << flush;
                    flags.no_flags = false;
                    break;
                }
                case 'i': {
                    try { flags.device_id = stol(optarg); }
                    catch (...) { cout <<  "Ошибка: некорректное значение ID" << endl; return -1; }
                    flags.no_flags = false;
                    break;
                }
                case 'n': {
                    flags.on = true;
                    flags.no_flags = false;
                    break;
                }
                case 'f': {
                    flags.off = true;
                    flags.no_flags = false;
                    break;
                }
                case 'r': {
                    flags.read = true;
                    flags.no_flags = false;
                    break;
                }
                case 'a': {
                    flags.all = true;
                    flags.no_flags = false;
                    break;
                }
                case 'p': {
                    try { 
                        flags.pulse_width = stoi(optarg);
                        if ( (flags.pulse_width > 255) || (flags.pulse_width < 1) )
                            {
                              cout <<  "Ошибка: некорректное значение ключа --pulse" << endl;
                              return -1;
                            }
                        }
                        catch (...) { cout <<  "Ошибка: некорректное значение ключа --pulse" << endl; return -1; }
                    flags.no_flags = false;
                    break;
                }
                case 'd': {
                    try {
                         flags.pulse_delay = stoi(optarg);
                         if ( (flags.pulse_delay ==0) || (flags.pulse_delay > 65535) )
                            {
                             cout <<  "Ошибка: некорректное значение ключа --delay" << endl;
                             return -1;
                            }
                        }
                        catch (...) { cout <<  "Ошибка: некорректное значение ключа --delay" << endl; return -1; }
                    flags.no_flags = false;
                    break;
                }
                case '?': default: {
                    cout << "Неизвестный параметр" << endl;
                    break;
                }
            }
        }

    if  (
        ( (flags.on)  && (flags.off) )          ||
        ( (flags.all) && (flags.device_id) )  
        )
        { cout << "Неверно заданы параметры\n" << endl; MAIN_RESULT = -1; }
    else
    {
        //  Подключение к RODOS-3 и исполнение команд
        struct hid_device_info *devs;
        int device_counter = 0;

        devs = hid_enumerate(R3_VID, R3_PID);

        if (flags.no_flags) cout << "Поиск устройств..." << endl;
        
        RODOS3_t *R3 = NULL;
        while (devs)
        {
            try { R3 = new RODOS3_t(devs->path); }
            catch (...) { cout << "Memory allocation failure!" << endl; }
            
            if (R3->CHECK_HANDLE() == false) cout << "Ошибка подключения" << endl;
            else
            {
                wstring PrString;
                R3->Read_Device_Production_String(PrString);

                if (flags.no_flags) {
                    wcout << PrString << flush;
                    cout << " ID: " << R3->Get_Device_ID() << endl;
                }
                if ( (PrString.size() >= 7) && (!wcscmp(PrString.substr(0, 7).c_str(), L"RODOS-3")) )
                {
                    device_counter++;
                    if (((flags.device_id == 0)&&(flags.all)) || (flags.device_id == R3->Get_Device_ID()))
                    {
                        if (flags.on)
                        {
                            R3->STATE__ON();
                            cout << "ID: " << R3->Get_Device_ID() << " " << std::string(PrString.begin(), PrString.end()) << ": реле включено" << endl;
                        }
                        if (flags.off)
                        {
                            R3->STATE__OFF();
                            cout << "ID: " << R3->Get_Device_ID() << " " << std::string(PrString.begin(), PrString.end()) << ": реле выключено" << endl;
                        }
                        if (flags.read)
                        {
                            uint8_t PS;
                            if (R3->STATE__READ(&PS))
                            {
                                if (PS==0x19)  { cout << "ID: " << R3->Get_Device_ID() << " " << std::string(PrString.begin(), PrString.end()) << ": реле выключено" << endl; }
                                else  	       { cout << "ID: " << R3->Get_Device_ID() << " " << std::string(PrString.begin(), PrString.end()) << ": реле включено"  << endl; }
                            }
                            else { cout << "ID: " << R3->Get_Device_ID() << " " << std::string(PrString.begin(), PrString.end()) << ": Ошибка чтения" << endl; MAIN_RESULT = -1; }
                        }
                        if (flags.pulse_width > 0)
                        {
                            if ( R3 -> SET_PULSE(flags.pulse_delay, flags.pulse_width) )
                                {
                                 cout << "ID: " << R3->Get_Device_ID() << " " << std::string(PrString.begin(), PrString.end()) << ": PULSE COMMAND OK" << endl;
                                }
                                else { cout << "ID: " << R3->Get_Device_ID() << " " << std::string(PrString.begin(), PrString.end()) << ": PULSE COMMAND ERR" << endl; }
                        }
                    }
                }
            }
            devs = devs->next;
            delete R3;
            R3 = NULL;
        }
        if (flags.no_flags) cout << "Найдено RODOS-3: " << device_counter << endl;
    }
    return MAIN_RESULT;
}
//---------------------------------------------------------------------------------------------------
void flags_t::init(void)
{
    no_flags = true;
    on = false;
    off = false;
    all = false;
    read = false;
    
    device_id = 0;
    pulse_delay = 1;
    pulse_width = 0;
}
//---------------------------------------------------------------------------------------------------
