#include "usb.h"
#include <iostream>
#include <unistd.h>
//------------------------------------------------------------
usb_t::usb_t(uint16_t VID, uint16_t PID)
{
    handle = hid_open(VID, PID, NULL);
    if (handle)
    {
        hid_get_product_string (handle, Product_String, PRODUCT_STRING_MAX_BUF_SIZE);
        USB_GET_ID(&Device_ID);
    }
}
//------------------------------------------------------------
usb_t::usb_t(const char *hid_device_path)
{
    handle = hid_open_path(hid_device_path);
    if (handle)
    {
        hid_get_product_string (handle, Product_String, PRODUCT_STRING_MAX_BUF_SIZE);
        USB_GET_ID(&Device_ID);
    }
}
//------------------------------------------------------------
usb_t::~usb_t() {
    if (handle != NULL) hid_close(handle);
}
//------------------------------------------------------------
void usb_t::Sleep(int ms)
{
    usleep (ms*1000);
}
//------------------------------------------------------------
bool usb_t::CHECK_HANDLE() {
    if (handle == NULL) return false;
    else return true;
}
//------------------------------------------------------------
void usb_t::USB_BUF_CLEAR()
{   //  очистка буферов приёма и передачи
    for (int i=0; i<9; i++) { USB_BUFI[i]=0; USB_BUFO[i]=0; }
}
//------------------------------------------------------------
bool usb_t::USB_GET_FEATURE()
{   //  чтение в буфер из устройства
    for ( uint8_t TryCount = 3; TryCount > 0; TryCount-- )
        {
         try {
              if (hid_get_feature_report(handle, USB_BUFI, 9) != -1) { return true; }
             }
             catch (...) { }
        }
    return false;
}
//------------------------------------------------------------
bool usb_t::USB_SET_FEATURE()
{   //  запись из буфера в устройство
    try { hid_send_feature_report(handle, USB_BUFO, 9); }
    catch (...) { return false; }
    return true;
}
//------------------------------------------------------------
bool usb_t::USB_GET_PORT(uint8_t *PS)
{   //  чтение состояния порта, 2ms
    USB_BUF_CLEAR();
    USB_BUFO[1]=0x7E;
    for (uint8_t TryCount = 3; TryCount > 0; TryCount--)
        {
        if ( USB_SET_FEATURE() && USB_GET_FEATURE() && (USB_BUFI[1]==0x7E) && (USB_BUFI[2]==USB_BUFI[3]) )
            {
             *PS=USB_BUFI[2];
             return true;
            }
        }
    return false;
}
//------------------------------------------------------------
bool usb_t::USB_SET_PORT(uint8_t PS)
{   //  запись состояния порта, 2ms
    USB_BUF_CLEAR();
    USB_BUFO[1]=0xE7;
    USB_BUFO[2]=PS;
    for (uint8_t TryCount = 3; TryCount > 0; TryCount-- )
        {
         if ( USB_SET_FEATURE() && USB_GET_FEATURE() && 
            (USB_BUFI[1]==0xE7) && 
            (USB_BUFI[2]==PS)   &&
            (USB_BUFI[3]==PS)
            )
            { return true; }
        }
    return false;
}
//------------------------------------------------------------
bool usb_t::USB_GET_FAMILY(uint8_t *FAMILY)
{   //  чтение группового кода устройства, 2ms
    USB_BUF_CLEAR();
    USB_BUFO[1]=0x1D;
    for (uint8_t TryCount = 3; TryCount > 0; TryCount--)
        {
         if ( USB_SET_FEATURE() && USB_GET_FEATURE() && (USB_BUFI[1]==0x1D) )
            {
             *FAMILY=USB_BUFI[2];
             return true;
            }
        }
    return false;
}
//------------------------------------------------------------
bool usb_t::USB_GET_SOFTV(uint16_t *SV)
{   //  чтение номера версии прошивки, 2ms
    USB_BUF_CLEAR();
    USB_BUFO[1]=0x1D;
    for (uint8_t TryCount = 3; TryCount > 0; TryCount--)
        {
         if ( USB_SET_FEATURE() && USB_GET_FEATURE() && (USB_BUFI[1]==0x1D) )
            {
             *SV = USB_BUFI[3] | ((uint16_t)USB_BUFI[4]<<8);
             return true;
            }
        }
    return false;
}
//------------------------------------------------------------
bool usb_t::USB_GET_ID(uint32_t *ID)
{   //  чтение ID устройства, 2ms
    USB_BUF_CLEAR();
    bool RESULT=false;
    USB_BUFO[1]=0x1D;
    for (uint8_t TryCount = 3; TryCount > 0; TryCount--)
        {
         if ( USB_SET_FEATURE() && USB_GET_FEATURE() && (USB_BUFI[1]==0x1D) )
            {
             *ID = ((uint32_t)USB_BUFI[5]<<24) | ((uint32_t)USB_BUFI[6]<<16) | ((uint32_t)USB_BUFI[7]<<8) | USB_BUFI[8];
             return true;
            }
        }
    return false;;
}
//------------------------------------------------------------
uint32_t usb_t::Get_Device_ID (void)
{
    return Device_ID;
}
//------------------------------------------------------------
void usb_t::Read_Device_Production_String (std::wstring &wString)
{
    for (int i=0; i<wcslen(Product_String); i++) wString += Product_String[i];
}
//------------------------------------------------------------
bool usb_t::USB_TIMERS(uint16_t S, uint8_t D)
{   //  запись таймеров
    USB_BUF_CLEAR();
    bool RESULT=false;
    uint8_t S_LSB = (uint8_t)S;
    uint8_t S_MSB = (uint8_t)(S>>8);
    USB_BUFO[1]=0x5A;
    USB_BUFO[2]=S_LSB;
    USB_BUFO[3]=S_MSB;
    USB_BUFO[4]=D;
    for (uint8_t TryCount = 3; TryCount > 0; TryCount-- )
        {
         if (
             USB_SET_FEATURE() &&
             USB_GET_FEATURE() &&
            (USB_BUFI[1]==0x5A)  &&
            (USB_BUFI[2]==S_LSB) &&
            (USB_BUFI[3]==S_MSB) &&
            (USB_BUFI[4]==D)
            )
            { return true; }
        }
    return false;
}
//------------------------------------------------------------
